#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

////////////////////////////////////////////////////////////////////////////////
// Process an array of OptN options on GPU
////////////////////////////////////////////////////////////////////////////////
#define BLOCK_N 140
#include "binomialOptions_kernel.cu"

void binomialOptionModel(
    double *h_CallResult,
    double *h_StockPrice,
    double *h_OptionStrike,
    double *h_OptionYears,
    double Riskfree,
    double Volatility,
    int OptN)
{
    int OPT_SZ = OptN * sizeof(double);

    double *d_StockPrice;
    double *d_OptionStrike;
    double *d_OptionYears;
    double *d_CallResult;

    cudaMalloc((void **)&d_StockPrice,   OPT_SZ);
    cudaMalloc((void **)&d_OptionStrike, OPT_SZ);
    cudaMalloc((void **)&d_OptionYears,  OPT_SZ);
    cudaMalloc((void **)&d_CallResult,   OPT_SZ);

    cudaMemcpy(d_StockPrice,   h_StockPrice,    OPT_SZ, cudaMemcpyHostToDevice);
    cudaMemcpy(d_OptionStrike, h_OptionStrike,  OPT_SZ, cudaMemcpyHostToDevice);
    cudaMemcpy(d_OptionYears,  h_OptionYears,   OPT_SZ, cudaMemcpyHostToDevice);

    cudaThreadSynchronize();
    binomialOptionsGPU<<<BLOCK_N, 256>>>(
            d_CallResult,
            d_StockPrice,
            d_OptionStrike,
            d_OptionYears,
            Riskfree,
            Volatility,
            OptN
        );

     cudaThreadSynchronize();

     cudaMemcpy(h_CallResult, d_CallResult, OPT_SZ, cudaMemcpyDeviceToHost);

     cudaFree(d_CallResult);
     cudaFree(d_OptionYears);
     cudaFree(d_OptionStrike);
     cudaFree(d_StockPrice);

}


////////////////////////////////////////////////////////////////////////////////
// Helper function, returning uniformly distributed
// random double in [low, high] range
////////////////////////////////////////////////////////////////////////////////
double RandDouble(double low, double high){
    double t = (double)rand() / (double)RAND_MAX;
    return (1.0f - t) * low + t * high;
}

const int OPT_N  = 1200;

int main(int argc, char* argv[])
{
   double h_StockPrice[OPT_N];
   double h_OptionStrike[OPT_N];
   double h_OptionYears[OPT_N];
   double h_CallResult[OPT_N];
   double h_riskfree   = 0.02f;
   double h_volatility = 0.30f;


   // Generate options data
   int i = 0;
   for(i = 0; i < OPT_N; i++)
   {
       h_StockPrice[i]    = RandDouble(5.0f, 30.0f);
       h_OptionStrike[i]  = RandDouble(1.0f, 100.0f);
       h_OptionYears[i]   = RandDouble(0.25f, 10.0f);
       h_CallResult[i]    = -1.0f;
   }

   int x;
   for(x = 0; x < 100; x++) //loop to simulate load
   {
      // call into model
      binomialOptionModel(h_CallResult, h_StockPrice, h_OptionStrike,
                          h_OptionYears, h_riskfree, h_volatility, OPT_N);

      printf("loop %d\n", x);
   }
   return 0;
}

