#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

extern "C"
void binomialOptionModel(
    double *h_CallResult,
    double *h_StockPrice,
    double *h_OptionStrike,
    double *h_OptionYears,
    double Riskfree,
    double Volatility,
    int OptN
);


////////////////////////////////////////////////////////////////////////////////
// Process an array of OptN options on GPU
////////////////////////////////////////////////////////////////////////////////
#define BLOCK_N 140
#include "binomialOptions_kernel.cu"


extern "C"
void binomialOptionModel(
    double *h_CallResult,
    double *h_StockPrice,
    double *h_OptionStrike,
    double *h_OptionYears,
    double Riskfree,
    double Volatility,
    int OptN)
{
    int OPT_SZ = OptN * sizeof(double);

    double *d_StockPrice;
    double *d_OptionStrike;
    double *d_OptionYears;
    double *d_CallResult;

    cudaMalloc((void **)&d_StockPrice,  OPT_SZ);
    cudaMalloc((void **)&d_OptionStrike, OPT_SZ);
    cudaMalloc((void **)&d_OptionYears,  OPT_SZ);
    cudaMalloc((void **)&d_CallResult,   OPT_SZ);

    cudaMemcpy(d_StockPrice,   h_StockPrice,   OPT_SZ, cudaMemcpyHostToDevice);
    cudaMemcpy(d_OptionStrike, h_OptionStrike,  OPT_SZ, cudaMemcpyHostToDevice);
    cudaMemcpy(d_OptionYears,  h_OptionYears,   OPT_SZ, cudaMemcpyHostToDevice);

    cudaThreadSynchronize();
    binomialOptionsGPU<<<BLOCK_N, 256>>>(
            d_CallResult,
            d_StockPrice,
            d_OptionStrike,
            d_OptionYears,
            Riskfree,
            Volatility,
            OptN
        );

    cudaThreadSynchronize();

    cudaMemcpy(h_CallResult, d_CallResult, OPT_SZ, cudaMemcpyDeviceToHost);

    cudaFree(d_CallResult);
    cudaFree(d_OptionYears);
    cudaFree(d_OptionStrike);
    cudaFree(d_StockPrice);

}

